import os
import shutil
import filecmp
import warnings

import paste.script.command

here_cross = lambda x: os.path.join(os.path.abspath(os.path.dirname(__file__)), *x)

def paster(cmd):
    print "paster %s" % cmd
    from paste.script import command
    args = cmd.split()
    options, args = command.parser.parse_args(args)
    options.base_parser = command.parser
    command.system_plugins.extend(options.plugins or [])
    commands = command.get_commands()
    command_name = args[0]
    if command_name not in commands:
        command = command.NotFoundCommand
    else:
        command = commands[command_name].load()
    runner = command(command_name)
    runner.run(args[1:])

def listdir(path):
    """Like os.listdir with '.svn' filter"""

    return [f for f in os.listdir(path) if f not in (('.svn',))]

def isFolderContentEqual(a, b):
    len_a = len(a)
    for root, dirs, files in os.walk(a):
        folder_part = root[len_a:].strip(os.path.sep)
        
        if not os.path.exists(os.path.join(b, folder_part)):
            warnings.warn("%s not exists" % (os.path.join(b, folder_part)))
            return False

        # Compare files and directory
        if listdir(root) != listdir(os.path.join(b, folder_part)):
            warnings.warn("%s != %s" % (listdir(root), listdir(os.path.join(b, folder_part))))
            return False
        
        # Compare files contents
        for f in files:
            if not filecmp.cmp(
                os.path.join(b, folder_part, f),
                os.path.join(root, f)
            ):
                warnings.warn("%s != %s" % (os.path.join(b, folder_part, f), os.path.join(root, f)))
                return False

    return True

def createPackage(package_name, buildout):
    tmp = here_cross(('tmp',))

    if os.path.isdir(tmp):
        shutil.rmtree(tmp)

    cmd = 'create -t advanced_package %s package=%s version= description= long_description= keywords= author= author_email= url= license_name= zip_safe= buildout=%s -o %s' % (package_name, package_name, buildout, tmp)
    paster(cmd)


    assert isFolderContentEqual(
        os.path.join(tmp, package_name),
        here_cross((package_name, ))
    )

    if os.path.isdir(tmp):
        shutil.rmtree(tmp)

def testMyNameSpace():
    createPackage(package_name = "my.name.space", buildout = "")

def testSimpleWithBuildout():
    createPackage(package_name = "simple_with_buildout", buildout = "True")



import os
import shutil

from paste.util.template import paste_script_template_renderer
from paste.script.templates import Template, var

class AdvancedPackageTemplate(Template):
    _template_dir = 'templates'
    summary = "Advanced setuptools-enabled package with buildout and namespace support"
    vars = [
        var('version', 'Version (like 0.1)'),
        var('description', 'One-line description of the package'),
        var('long_description', 'Multi-line description (in reST)'),
        var('keywords', 'Space-separated keywords/tags'),
        var('author', 'Author name'),
        var('author_email', 'Author email'),
        var('url', 'URL of homepage'),
        var('license_name', 'License name'),
        var('zip_safe', 'True/False: if the package can be distributed as a .zip file', default=False),
        var('buildout', 'True/False: buildout support', default=False),
        ]

    template_renderer = staticmethod(paste_script_template_renderer)

    def pre(self, command, output_dir, vars):
        if vars['package'].find('.') > 0:
            vars['namespace'] = vars['package'].split('.')
        else:
            vars['namespace'] = [ vars['package'] ]

    def post(self, command, output_dir, vars):
        template_dir = self.template_dir()
        current_dir = os.path.join(output_dir, "src")
        for part in vars['namespace'][:-1]:
            current_dir = os.path.join(current_dir, part)
            if not os.path.exists(current_dir):
                os.makedirs(current_dir)

            shutil.copyfile(
                os.path.join(template_dir, '.namespace__init__.py'),
                os.path.join(current_dir, '__init__.py')
            )
        
        part = vars['namespace'][-1]
        current_dir = os.path.join(current_dir, part)
        if not os.path.exists(current_dir):
            os.makedirs(current_dir)

        shutil.copyfile(
            os.path.join(template_dir, '.__init__.py'),
            os.path.join(current_dir, '__init__.py')
        )

        if (
            (vars['buildout'] == False) or
            (vars['buildout'] == "False") or
            (vars['buildout'] == "") 
        ):
            os.remove(os.path.join(output_dir, "buildout.cfg"))
            os.remove(os.path.join(output_dir, "bootstrap.py"))



from setuptools import setup, find_packages
import sys, os

version = '0.0'

setup(name='my.name.space',
      version=version,
      description="",
      long_description="""\
""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='',
      author='',
      author_email='',
      url='',
      license='',
      packages=find_packages('src'),
      package_dir = {'': 'src'},
      namespace_packages = ['my', 'my.name'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import sys, os

version = '0.2'

setup(name='harobed.paster_template.advanced_package',
      version=version,
      description="PasteScript template to create advanced setuptools-enabled package with buildout and namespace support",
      long_description="""\

Features supports
=================

* namespace support (handle multiple namespaces levels)
* buildout support

Install
=======

Install the template :

::

    $ sudo easy_install harobed.paster_template.advanced_package


Using
=====

Create a new package with advanced_package template :

::
    
    $ paster create -t advanced_package my.new.package package=my.new.package

You can answer True to buildout question to install buildout files (bootstrap.py and buildout.cfg).
""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='',
      author='KLEIN Stephane',
      author_email='stephane@harobed.org',
      url='http://code.google.com/p/paste-script-advanced-package-template/',
      license='GNU General Public License v3',
      packages = find_packages('src'),
      package_dir = {'':'src'},
      namespace_packages = ['harobed', 'harobed.paster_template'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
          "Paste",
          "PasteScript",
          "setuptools"
      ],
      entry_points="""
      [paste.paster_create_template]
      advanced_package = harobed.paster_template.advanced_package:AdvancedPackageTemplate
      """,
      )
